#ifndef CONNECT4_H
#define CONNECT4_H

#include <stdbool.h>

#define ROWS 6
#define COLUMNS 7
#define WINLEN 4

typedef unsigned char connect4_move; /* 1..COLUMNS */
typedef unsigned short connect4_moves_set; /* set of 1..COLUMNS */
enum connect4_slot {CONNECT4_FREE, CONNECT4_WHITE, CONNECT4_BLACK};
typedef struct {
   short filled; // number of tokens filled into the board so far
   bool finished; // true if someone has won or nobody can win
   enum connect4_slot winner;
   enum connect4_slot board[ROWS][COLUMNS];
   // how many tokens have been filled into a particular column
   short height[COLUMNS];
   // keep notice of last move and the total number of moves
   connect4_move last_move;
   unsigned int number_of_moves;
} connect4_board;

/* initialize board for the start of a new game; WHITE begins */
void connect4_init(connect4_board* board);

/* return true, if the game is finished */
bool connect4_finished(connect4_board* board);

/* return true, if the game is finished and nobody has won */
bool connect4_tied(connect4_board* board);

/* return CONNECT4_WHITE, if white has won, or
          CONNECT4_BLACK, if black has won */
enum connect4_slot connect4_winner(connect4_board* board);

/* return the next player, i.e. CONNECT4_WHITE or CONNECT4_BLACK
   PRE: the game is not yet finished */
enum connect4_slot connect4_next(connect4_board* board);

/* return true if this move is valid */
bool connect4_valid_move(connect4_board* board, connect4_move move);

/* perform the given move on the given board
   PRE: this move must be valid */
void connect4_perform_move(connect4_board* board, connect4_move move);

/* return the set of valid moves */
connect4_moves_set connect4_valid_moves(connect4_board* board);

/* return true if the game has already started,
   i.e. at least one move has been performed */
bool connect4_started(connect4_board* board);

/* return last move
   PRE: connect4_started returns true */
connect4_move connect4_last_move(connect4_board* board);

#endif
