#include "connect4.h"
#include "connect4_eval.h"
#include "connect4_stretches.h"

int connect4_eval(connect4_board* board, enum connect4_slot viewpoint) {
   if (connect4_finished(board)) {
      if (connect4_tied(board)) {
	 return 0;
      } else if (connect4_winner(board) == viewpoint) {
	 return INFINITELY_GOOD;
      } else {
	 return INFINITELY_BAD;
      }
   }
   enum connect4_slot nextplayer = connect4_next(board);

   int evaluation = 0;
   /* check the individual stretches */
   for (int si = 0; si < sizeof stretch / sizeof(struct stretch); ++si) {
      int i = stretch[si].i; int j = stretch[si].j;
      int di = stretch[si].di; int dj = stretch[si].dj;
      enum connect4_slot last_token = CONNECT4_FREE;
      unsigned char len = 0;
      unsigned char white_len = 0;
      unsigned char token_len = 0;
      unsigned char before_white_len = 0;
      for (int index = 0; index < stretch[si].len; ++index) {
	 enum connect4_slot token = board->board[i][j];
	 if (token != CONNECT4_FREE) {
	    if (token == last_token) {
	       ++len; ++token_len;
	    } else {
	       len = white_len + 1;
	       token_len = 1;
	       last_token = token;
	       before_white_len = white_len;
	       white_len = 0;
	    }
	 } else {
	    ++white_len;
	    if (last_token != CONNECT4_FREE) ++len;
	 }
	 if (len >= WINLEN) {
	    int value;
	    if (last_token == viewpoint) {
	       value = token_len;
	    } else {
	       value = -token_len;
	    }
	    if (token_len == WINLEN-1) {
	       value *= 100;
	       if (last_token == nextplayer) {
		  value *= 200;
	       }
	       if (before_white_len > 0 && white_len > 0) {
		  value *= 200;
	       }
	    } else if (token_len == WINLEN-2 &&
		  before_white_len > 0 && white_len > 0) {
	       value *= 100;
	       if (last_token == nextplayer) value *= 200;
	    }
	    evaluation += value;
	 }
	 i += di; j += dj;
      }
   }
   if (evaluation < INFINITELY_BAD) evaluation = INFINITELY_BAD;
   if (evaluation > INFINITELY_GOOD) evaluation = INFINITELY_GOOD;
   return evaluation;
}

int connect4_recursive_eval(connect4_board board, connect4_move move,
      int level, enum connect4_slot viewpoint) {
   connect4_perform_move(&board, move);
   int evaluation = connect4_eval(&board, viewpoint);
   if (level == 0 ||
	 evaluation == INFINITELY_BAD || evaluation == INFINITELY_GOOD) {
      return evaluation;
   }
   --level;

   enum connect4_slot next_player = connect4_next(&board);
   connect4_moves_set moves = connect4_valid_moves(&board);
   int best_value = INFINITELY_BAD;
   for (int move = 1; move <= COLUMNS; ++move) {
      if (moves & 1<<(move-1)) {
	 int eval = connect4_recursive_eval(board, move, level, next_player);
	 if (eval > best_value) {
	    best_value = eval;
	 }
      }
   }
   if (next_player == viewpoint) {
      return best_value;
   } else {
      return -best_value;
   }
}
