#ifndef CONNECT4_PACKETS
#define CONNECT4_PACKETS

#include <stdbool.h>
#include <stralloc.h>
#include <afblib/outbuf.h>
#include <afblib/inbuf.h>

#define CONNECT4_PORT 59059

enum connect4_command {
   CONNECT4_LOGIN, /* UserName -- command to start a session */
   CONNECT4_USERS, /* -- list all registered users that are ready to play */
   CONNECT4_PLAY, /* [UserName] -- wait for someone or invite someone */
   CONNECT4_MOVE, /* Move -- perform a move */
   CONNECT4_QUIT, /* -- leave */
};

/* please note that the char pointers in the following two structures
   are to be freed as soon as they are no longer needed,
   i.e. those who are calling parse_request or parse_response
   are responsible for freeing them;
   likewise callers of gen_response_packet, gen_response_packet,
   or gen_request are responsible for freeing the character strings */

typedef struct connect4_request {
   enum connect4_command command;
   char* param; /* may be 0, or username, or move */
} connect4_request;

typedef struct connect4_response {
   bool succeeded;
   char* result; /* may be 0 if there is no response beyond OK/FAILURE */
} connect4_response;

/* server side */
bool parse_request(stralloc* s, connect4_request* request);
bool gen_response(connect4_response* response, stralloc* s);
bool gen_response_packet(bool succeeded, stralloc* msg, stralloc* s);

/* client side */
bool gen_request(outbuf* obuf, connect4_request* request);
bool parse_response(inbuf* ibuf, connect4_response* response);

#endif
