#include <stdio.h>
#include <stdlib.h>
#include <afblib/hostport.h>
#include <afblib/strlist.h>
#include <afblib/inbuf_readline.h>
#include "connect4.h"
#include "connect4_client.h"

void show_board(connect4_board* board) {
   printf("   +");
   for (int col = 0; col < COLUMNS; ++col) {
      putchar('-'); putchar('-');
   }
   printf("-+\n");
   for (int row = ROWS-1; row >= 0; --row) {
      printf("%2d |", row+1);
      for (int col = 0; col < COLUMNS; ++col) {
	 putchar(' ');
	 switch (board->board[row][col]) {
	    case CONNECT4_FREE:
	       putchar(' '); break;
	    case CONNECT4_WHITE:
	       putchar('X'); break;
	    case CONNECT4_BLACK:
	       putchar('O'); break;
	 }
      }
      printf(" |\n");
   }
   printf("   +");
   for (int col = 0; col < COLUMNS; ++col) {
      putchar('-'); putchar('-');
   }
   printf("-+\n");
   printf("    ");
   for (int col = 0; col < COLUMNS; ++col) {
      printf("%2d", col+1);
   }
   putchar('\n');
}

bool read_move(inbuf* ibuf, connect4_board* board, connect4_move* movep) {
   for(;;) {
      printf("Your move: "); fflush(stdout);
      char* line = inbuf_readline(ibuf);
      if (line == 0) return false;
      int move = atoi(line); free(line);
      if (move < 1 || move > COLUMNS) {
	 printf("Bad move, please give a number between 1 and %d\n", COLUMNS);
	 continue;
      }
      if (!connect4_valid_move(board, move)) {
	 printf("Invalid move, please try again.\n");
	 continue;
      }
      *movep = move;
      return true;
   }
}

int main(int argc, char** argv) {
   char* cmdname = *argv++; --argc;
   if (argc != 3) {
      fprintf(stderr, "Usage: %s hostport login opponent\n", cmdname);
      exit(1);
   }
   char* hport = *argv++; --argc;
   char* login = *argv++; --argc;
   char* opponent = *argv++; --argc;

   hostport hp = {0};
   if (!parse_hostport(hport, &hp, CONNECT4_PORT)) {
      perror(hport);
      exit(1);
   }
   int fd;
   if ((fd = socket(hp.domain, SOCK_STREAM, hp.protocol)) < 0) {
      perror("socket"); exit(1);
   }
   if (connect(fd, (struct sockaddr *) &hp.addr, hp.namelen) < 0) {
      perror(hport); exit(1);
   }

   struct connect4_service service = {0};
   c4_init(&service, fd);

   if (!c4_login(&service, login)) {
      fprintf(stderr, "Unable to login to connect4 service\n");
      c4_hangup(&service);
      exit(1);
   }

   connect4_board board = {0};
   if (!c4_invite(&service, opponent, &board)) {
      fprintf(stderr, "Unable to play with %s\n", opponent);
      c4_quit(&service);
      c4_hangup(&service);
      exit(1);
   }

   enum connect4_slot me;
   printf("Game with %s starts, you have ", opponent);
   if (connect4_started(&board)) {
      printf("black.\n");
      printf("Your tokens are marked with Os, those of %s with Xs.\n",
         opponent);
      printf("The initial move of %s is %d:\n", opponent,
	 connect4_last_move(&board));
      me = CONNECT4_BLACK;
   } else {
      printf("white.\n");
      printf("Your tokens are marked with Xs, those of %s with Os.\n",
         opponent);
      printf("You start, the board is empty:\n");
      me = CONNECT4_WHITE;
   }
   inbuf stdinbuf = {0};
   for(;;) {
      show_board(&board);
      connect4_move move;
      if (!read_move(&stdinbuf, &board, &move)) break;
      if (!c4_move(&service, &board, move)) {
	 printf("Sorry, this game was unfortunately aborted!\n"); break;
      }
      if (connect4_finished(&board)) break;
      printf("Your opponent's move is %d:\n", connect4_last_move(&board));
   }
   if (connect4_finished(&board)) {
      printf("Final board:\n"); show_board(&board);
      if (connect4_tied(&board)) {
	 printf("Tied!\n");
      } else if (connect4_winner(&board) == me) {
	 printf("You won!\n");
      } else {
	 printf("You lost!\n");
      }
   }
   printf("Bye!\n");

   c4_quit(&service);
   c4_hangup(&service);
}
